using OffsetArrays
using Plots

function scaled_bernoulli(T, n)
    facto = OffsetArray([one(T), one(T)], OffsetArrays.Origin(0))
    pow4 = OffsetArray([one(T)], OffsetArrays.Origin(0))
    b = OffsetArray([one(T)], OffsetArrays.Origin(0))
    for k = 1:n
        push!(facto, facto[end]*(2k))
        push!(facto, facto[end]*(2k+1))
        push!(pow4, pow4[end]*4)
        s = zero(T)
        for j = 0:k-1
            s += b[j]/(facto[2k+1-2j]*pow4[k-j])
        end
        push!(b, 1/(facto[2k]*pow4[k]) - s)
    end
    return b
end

n = 500
setprecision(53)
val = scaled_bernoulli(BigFloat, n)
ref = scaled_bernoulli(Rational{BigInt}, n)
err = abs.((convert.(Rational{BigInt}, val) - ref) ./ ref)
u = eps(BigFloat)
bound(k) = (1 + 21.2*u)^k * (1.1*k + 446)*u

plot([err, reshape(bound.(0:n), 0:n)] ./ u,
     leg=false,
     size=(300, 120),
     xlabel="terms",
     ylabel="max. relative error",
     tickfont=font(4, "times"),
     guidefont=font(5, "times")
    )
Plots.savefig("bernoulli.pdf")
