#!/usr/bin/julia

using Plots
using Measures

function doit(c0, n, stride)
    pre = zero(Float32)
    cur = ini = Float32(c0)
    data = []
    u = Float64(eps(Float32))
    alpha = 1 + u + sqrt(u*(1+u))
    for i in 0:n
        if i % stride == 0
            err = Float64(cur)/((i+1)*Float64(ini)) - 1
            # push!(data, [log2(abs(err)/u), log2((i+1)*(i+2)*(i+3)/6*alpha^i)])
            push!(data, [abs(err)])
        end
        pre, cur = cur, 2*cur - pre
    end
    vcat(reshape.(data, 1, :)...)
end

nini = 997
n = 100_000
stride = 9973
data = hcat(doit.(range(pi/2, pi, length=nini), n, stride)...)
default(
        leg=false,
        size=(380, 120),
        tickfontsize=4,
        labelfontsize=5,
        xlabel="terms",
        ylabel="max. relative error",
        tickfont="times",
        guidefontfamily="times"
       )
p1 = plot(0:stride:n, [maximum(r) for r in eachrow(data)])
nini = 997
n = 40_000_000
stride = 9973
data = hcat(doit.(range(pi/2, pi, length=nini), n, stride)...)
p2 = plot(0:stride:n, [maximum(r) for r in eachrow(data)])
p = plot(p1, p2, layout=2)
Plots.savefig("toy.pdf")
